/** Model */
open multi                                  // a utility module providing the function drop

open mrel[Bundle, Item] as Contains          // declare a multirelation from Bundle to Item
open mrel[Item, Category] as BelongsTo     // declare a multirelation from Item to Category
open mrel[Bundle, Category] as Result             // to be the result of Contains;BelongsTo

abstract sig Bundle {}
one sig B1, B2 extends Bundle {}                         // singletons to represent bundles
abstract sig Item {
  belongsTo: set Category             // declare an ordinary relation from Item to Category
}
one sig Bread, Butter, Milk extends Item {}
fact {
  Bread.belongsTo = Bakery                          // assign the items to their categories
  Butter.belongsTo = Dairy
  Milk.belongsTo = Dairy
}
abstract sig Category {}
one sig Dairy, Bakery extends Category {}     // singletons to represent product categories

fact {
  BelongsTo/liftedFrom[belongsTo]          // lift the ordinary relation to a multirelation
  Result/composedFrom[Contains/get, BelongsTo/get]                 // perform the multijoin
  // rules
  all b : Bundle | #(b<:Result/get:>Dairy) >= 2              // at least two dairy products
  all b : Bundle | #(b.(drop[Result/get])) >= 2                  // at least two categories
}

assert AllHaveBread {               // based on our rules, all bundles should contain bread
  all b: Bundle | some drop[b<:Contains/get].Bread
}


/** Visualization  Guide*/

/*
  To visualize a multi-relation,
  Firstly set up a function which returns a ternary relation
  In the function body, invoke the `get` function of the mrel you want to visualize.
  Then in the Alloy visualizer, click `Theme` on the menu bar,
  in the left panel, uncheck `show` for every signature whose name ends up with `Head`
  and uncheck `show` for every relation expect the relations created by the below functions
*/

fun dispContains[]: univ->univ->univ { Contains/get }
fun dispBelongsTo[] : univ->univ->univ { BelongsTo/get }


run {} for 8   // instance finding, find all bundlings which conform to the rules
check AllHaveBread for 8  // assertion checking, all bundles should contain bread
